//**************************************************************************************************
//* libreadplus: the command-line dialog utility.
//* readln: wrap GNU readline
//* (C) Joachim Wuttke 2001, 2012-
//* http://apps.jcns.fz-juelich.de
//**************************************************************************************************

#include <iostream>
#include <string>
#include <readline/readline.h>
#include <readline/history.h>

#include "readln.hpp"

using std::string;
using std::cout;

//**************************************************************************************************
//*  Key bindings
//**************************************************************************************************


namespace NReadln {
    int PF1( int c, int k );
    int PF2( int c, int k );
    int PF3( int c, int k );
    int PF4( int c, int k );
    int F05( int c, int k );
    int F06( int c, int k );
    int F07( int c, int k );
    int F08( int c, int k );
    int F09( int c, int k );
    int F10( int c, int k );
    int F11( int c, int k );
    int F12( int c, int k );
    int function_key( int num );
}


//! Executed when readline encounters a function key (PF1 ... F12).

int NReadln::function_key( int num )
{
    cout << "\n"; // the user did not terminate with \n, so we have to do it
    char out[5];
    sprintf( out, "FK%02i", num );
    rl_insert_text( out );
    rl_done = 1;
    rl_on_new_line ();
    return 0;
}

// Functions readln_PF1 etc just call readln_function_key.

int NReadln::PF1( int c, int k ) { return NReadln::function_key( 1); }
int NReadln::PF2( int c, int k ) { return NReadln::function_key( 2); }
int NReadln::PF3( int c, int k ) { return NReadln::function_key( 3); }
int NReadln::PF4( int c, int k ) { return NReadln::function_key( 4); }
int NReadln::F05( int c, int k ) { return NReadln::function_key( 5); }
int NReadln::F06( int c, int k ) { return NReadln::function_key( 6); }
int NReadln::F07( int c, int k ) { return NReadln::function_key( 7); }
int NReadln::F08( int c, int k ) { return NReadln::function_key( 8); }
int NReadln::F09( int c, int k ) { return NReadln::function_key( 9); }
int NReadln::F10( int c, int k ) { return NReadln::function_key(10); }
int NReadln::F11( int c, int k ) { return NReadln::function_key(11); }
int NReadln::F12( int c, int k ) { return NReadln::function_key(12); }


//! Bind function keys.

int NReadln::set_bindings()
{
    rl_bind_keyseq( "\\eOP",   NReadln::PF1 );
    rl_bind_keyseq( "\\eOQ",   NReadln::PF2 );
    rl_bind_keyseq( "\\eOR",   NReadln::PF3 );
    rl_bind_keyseq( "\\eOS",   NReadln::PF4 );
    rl_bind_keyseq( "\\e[15~", NReadln::F05 );
    rl_bind_keyseq( "\\e[17~", NReadln::F06 );
    rl_bind_keyseq( "\\e[18~", NReadln::F07 );
    rl_bind_keyseq( "\\e[19~", NReadln::F08 );
    rl_bind_keyseq( "\\e[20~", NReadln::F09 );
    rl_bind_keyseq( "\\e[21~", NReadln::F10 );
    rl_bind_keyseq( "\\e[23~", NReadln::F11 );
    rl_bind_keyseq( "\\e[24~", NReadln::F12 );
    return 0;
}


//**************************************************************************************************
//*  API
//**************************************************************************************************


//! Initializes readline and history.

void NReadln::initialize()
{
    using_history();
    NReadln::set_bindings();
}


//! Get interactive input via the readline/history API.

string NReadln::readline( const string& prompt )
{
    char *inp;
    string ret;
    inp = ::readline( prompt.c_str() ); // from libreadline
    if( inp && *inp )
        add_history( inp );  // from libhistory
    ret = inp;
    free( inp );
    return ret;
}

void NReadln::write_history( const string& fname )
{
    remove_history( where_history() );
    ::write_history( fname.c_str() );
    cout << "saved history to file '" << fname << "'\n";
}
